#include "stfu.h"
#include "cfg.h"
#include "lzr.h"
#include "input.h"
#include "font.h"
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static StfuCtx *ctx = NULL;

static void draw_box(int x, int y, int w, int h) {
	const int x0 = x - rand() % ctx->chaos;
	const int y0 = y - rand() % ctx->chaos;
	const int x1 = x - 1 + w + rand() % ctx->chaos;
	const int y1 = y - rand() % ctx->chaos;
	const int x2 = x - 1 + w + rand() % ctx->chaos;
	const int y2 = y - 1 + h + rand() % ctx->chaos;
	const int x3 = x - rand() % ctx->chaos;
	const int y3 = y - 1 + h + rand() % ctx->chaos;
	LZR_DrawLine(x0, y0, x1, y1);
	LZR_DrawLine(x1, y1, x2, y2);
	LZR_DrawLine(x2, y2, x3, y3);
	LZR_DrawLine(x3, y3, x0, y0);
}

int stfu_init(void)
{
	ctx = malloc(sizeof(StfuCtx));
	if (ctx == NULL)
		return 1;
	memset(ctx, 0, sizeof(StfuCtx));
	if (LZR_Init(cfg))
		return stfu_deinit(), 1;
	if (LZR_IMAGE(font.bmp))
		return stfu_deinit(), 1;
	ctx->chaos = 8;
	return 0;
}

void stfu_deinit(void)
{
	LZR_Quit();
	if (ctx != NULL) {
		memset(ctx, 0, sizeof(StfuCtx));
		free(ctx), ctx = NULL;
	}
}

void who(const char *fmt, ...)
{
	va_list valist;
	va_start(valist, fmt);
	vsnprintf(ctx->who, sizeof(ctx->who), fmt, valist);
	va_end(valist);
}

void say(const char *fmt, ...)
{
	va_list valist;
	va_start(valist, fmt);
	vsnprintf(ctx->say, sizeof(ctx->say), fmt, valist);
	va_end(valist);
	ctx->displayed = 0;
	ctx->state = STFU_SAY;
	show();
}

void show(void)
{
	for (;;) {
		const int invert = rand() % 40 + 1 < ctx->chaos;
		LZR_DrawBegin();
		LZR_DrawSetColor(invert, invert, invert);
		LZR_DrawClear();
		LZR_DrawSetColor(!invert, !invert, !invert);
		for (int i = 0; i < 2; i++)
			draw_box(16, CFG_DHEIGHT - 64, CFG_DWIDTH - 32, 48);
		font_draw(28, CFG_DHEIGHT - 48, ctx->say, ctx->displayed,
		          1 + (ctx->chaos - 1) / 2);
		LZR_DrawEnd();
		LZR_CycleEvents();
		input_update();
		if (LZR_GetTick() % 5 == 0 && ctx->say[ctx->displayed] != '\0')
			ctx->displayed += 1;
		if (LZR_ShouldQuit()) {
			stfu_deinit();
			exit(0);
		}
		if (input_pressed(K_O)) {
			if (ctx->say[ctx->displayed] != '\0')
				;//ctx->displayed = strlen(ctx->say);
			else
				break;
		}
	}
}

int ask(const char *c0, const char *c1, const char *c2)
{
	ctx->ask[0] = c0;
	ctx->ask[1] = c1;
	ctx->ask[2] = c2;
	ctx->displayed = 0;
	ctx->state = STFU_ASK;
	show();
	return ctx->choice;
}

void unreachable(void)
{
	stfu_deinit();
}

void chaos(int n)
{
	ctx->chaos = n;
}
